@@ -112,14 +112,17 @@ private:
 UT_uint32 				GR_CocoaGraphics::s_iInstanceCount = 0;
 
 bool      GR_CocoaGraphics::m_colorAndImageInited = false;
+
 NSImage * GR_CocoaGraphics::m_imageBlue16x15 = nil;
 NSImage * GR_CocoaGraphics::m_imageBlue11x16 = nil;
 NSImage * GR_CocoaGraphics::m_imageGrey16x15 = nil;
 NSImage * GR_CocoaGraphics::m_imageGrey11x16 = nil;
+
 NSColor * GR_CocoaGraphics::m_colorBlue16x15 = nil;
 NSColor * GR_CocoaGraphics::m_colorBlue11x16 = nil;
 NSColor * GR_CocoaGraphics::m_colorGrey16x15 = nil;
 NSColor * GR_CocoaGraphics::m_colorGrey11x16 = nil;
+
 NSCursor *	GR_CocoaGraphics::m_Cursor_E = nil;
 NSCursor *	GR_CocoaGraphics::m_Cursor_N = nil;
 NSCursor *	GR_CocoaGraphics::m_Cursor_NE = nil;
@@ -150,6 +153,7 @@ void GR_CocoaGraphics::_initColorAndImage(void)
 	if (m_colorAndImageInited) {
 		return;
 	}
+
 	if (path = [bundle pathForResource:@"Blue16x15" ofType:@"png"]) {
 		if (m_imageBlue16x15 = [[NSImage alloc] initWithContentsOfFile:path]) {
 			m_colorBlue16x15 = [NSColor colorWithPatternImage:m_imageBlue16x15];
@@ -180,7 +184,6 @@ void GR_CocoaGraphics::_initColorAndImage(void)
 		m_colorGrey16x15 = [NSColor grayColor];
 		[m_colorGrey16x15 retain];
 	}
-	
 	if (path = [bundle pathForResource:@"Grey11x16" ofType:@"png"]) {
 		if (m_imageGrey11x16 = [[NSImage alloc] initWithContentsOfFile:path]) {
 			m_colorGrey11x16 = [NSColor colorWithPatternImage:m_imageGrey11x16];
@@ -454,7 +457,7 @@ GR_CocoaGraphics::~GR_CocoaGraphics()
 	[m_fontProps release];
 	[m_fontForGraphics release];
 	[m_currentColor release];
-
+#if 0
 	RELEASEP(m_imageBlue16x15);
 	RELEASEP(m_imageBlue11x16);
 	RELEASEP(m_imageGrey16x15);
@@ -484,7 +487,7 @@ GR_CocoaGraphics::~GR_CocoaGraphics()
 	RELEASEP(m_Cursor_Crosshair);
 	RELEASEP(m_Cursor_HandPointer);
 	RELEASEP(m_Cursor_DownArrow);
-
+#endif
 	s_iInstanceCount--;
 	for (int i = 0; i < COUNT_3D_COLORS; i++) {
 		[m_3dColors[i] release];
@@ -665,7 +668,11 @@ void GR_CocoaGraphics::drawChars(const UT_UCSChar* pChars, int iCharOffset,
 								 int * pCharWidths)
 {
 	UT_DEBUGMSG (("GR_CocoaGraphics::drawChars()\n"));
-	UT_sint32 yoff = _tduY(yoffLU); // - m_pFont->getDescent();
+	//
+	// this is where things get tricky.
+	// AbiWord draws strings with the Y origin as top of the line while Cocoa do it 
+	// on the baseline
+	UT_sint32 yoff = _tduY(yoffLU + m_pFont->getAscent());
 	UT_sint32 xoff = xoffLU;	// layout Unit !
 
     if (!m_fontMetricsTextStorage) {
