@@ -33,42 +33,86 @@ class UT_ByteBuf;
 //////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////
 
-class GR_Win32Font : public GR_Font
+class ABI_EXPORT GR_Win32Font : public GR_Font
 {
 public:
-	GR_Win32Font(LOGFONT & lf);
+	static GR_Win32Font * newFont(LOGFONT & lf, double fPoints, HDC hdc, HDC printDC);
 	virtual ~GR_Win32Font();
 
-	// make GR_Win32Graphics an "aquaintance" of GR_Win32Font
-	class Acq
-	{
-		friend class GR_Win32Graphics;
-		friend class GR_Win32Font;
-	private:
+	// need these to allow for adjustements in response to changes of device
+	void    	setAscent(UT_uint32 n)  { m_tm.tmAscent = n; }
+	void 	    setDescent(UT_uint32 n) { m_tm.tmDescent = n; }
+	void        setHeight(UT_uint32 n)  { m_tm.tmHeight = n; }
+	
+	UT_uint32	getAscent(HDC hdc, HDC printHDC);
+	UT_uint32	getDescent(HDC hdc, HDC printHDC);
+	UT_uint32	getHeight(HDC hdc, HDC printHDC);
+	UT_uint32   getUnscaledHeight() const { return m_iHeight;}
 
-		static inline HFONT		getDisplayFont(GR_Win32Font& font, GR_Graphics * pGr);
-
-		static UT_uint32	measureUnRemappedChar(	GR_Win32Font& font,
-													UT_UCSChar c);
-		static UT_uint32	getAscent(const GR_Win32Font& font)
-			{ return font.m_tm.tmAscent; }
-		static UT_uint32	getDescent(const GR_Win32Font& font)
-			{ return font.m_tm.tmDescent; }
-		static UT_uint32	getFontHeight(const GR_Win32Font& font)
-			{ return font.m_tm.tmHeight; }
-
-		static void				selectFontIntoDC(GR_Win32Font& font, GR_Graphics * pGr, HDC hdc);
-	};
-	friend class Acq;
+	HFONT       getDisplayFont(GR_Graphics * pGr);
 
 	virtual UT_sint32 measureUnremappedCharForCache(UT_UCSChar cChar) const;
+	UT_sint32	measureUnRemappedChar(UT_UCSChar c);
 	virtual GR_CharWidths* newFontWidths(void) const;
+//
+// UT_Rect of glyph in Logical units.
+// rec.left = bearing Left (distance from origin to start)
+// rec.width = width of the glyph
+// rec.top = distance from the origin to the top of the glyph
+// rec.height = total height of the glyph
+
+	virtual bool glyphBox(UT_UCS4Char g, UT_Rect & rec, GR_Graphics * pG);
+	
+	void        selectFontIntoDC(GR_Graphics * pGr, HDC hdc);
+	
+	void        markGUIFont() {m_bGUIFont = true;}
+	bool        isFontGUI() const {return m_bGUIFont;}
+
+	const HDC   getPrimaryHDC() const {return m_hdc;}
+	const HDC   getXHDC() const {return m_xhdc;}
+	const HDC   getYHDC() const {return m_yhdc;}
+
+	void        setPrimaryHDC(const HDC hdc) {m_hdc = hdc;}
+	void        setXHDC(const HDC hdc) {m_xhdc = hdc;}
+	void        setYHDC(const HDC hdc) {m_yhdc = hdc;}
+	
+	
+	// NB: the font handle is one which was associated with this font when it was
+	// origianlly created; however, it is not necessarily one that is to be used for
+	// drawing as that has to reflect zoom factor and has to be obtained using
+	// GR_Win32Font::Acq::getDisplayFont()
+	// (The handle returned by getFontHandle() can be used for things that are not
+	// affected by zoom, such as retrieving face names, etc.)
+	HFONT       getFontHandle() const {return m_layoutFont;}
+	double      getPointSize() const {return m_fPointSize;}
 
 protected:
+	// all construction has to be done via the graphics class
+	GR_Win32Font(LOGFONT & lf, double fPoints, HDC hdc, HDC printHDC);
+
 	GR_Win32CharWidths * _getCharWidths() const
 	{
+#ifndef ABI_GRAPHICS_PLUGIN_NO_WIDTHS
 		return reinterpret_cast<GR_Win32CharWidths *>(GR_Font::_getCharWidths());
-	}	
+#else
+		UT_return_val_if_fail(UT_NOT_IMPLEMENTED,NULL);
+#endif
+	}
+
+	// this function should clear any cached information the font might cary
+	// it is prinicipally intened to be used when we share fonts between screen and
+	// printer 
+	virtual void _clearAnyCachedInfo() {};
+	void         _updateFontYMetrics(HDC hdc, HDC printHDC);
+
+
+public:
+	HFONT        getFontFromCache(UT_uint32 pixelsize, bool bIsLayout,
+								  UT_uint32 zoomPercentage) const;
+
+	void         fetchFont(UT_uint32 pixelsize) const;
+
+	const TEXTMETRIC & getTextMetric() const {return m_tm;}
    	
 private:
 
@@ -78,13 +122,20 @@ private:
 		HFONT			    hFont;
 	};
 
-	HFONT				    getFontFromCache(UT_uint32 pixelsize, bool bIsLayout, UT_uint32 zoomPercentage) const;
 	void					insertFontInCache(UT_uint32 pixelsize, HFONT pFont) const;
-	void					fetchFont(UT_uint32 pixelsize) const;
 
-	void					setupFontInfo();
-
-	HDC						m_oldHDC;
+	// we will store three different HDC values
+	// m_hdc is handle to the device on which we are meant to draw
+	// m_xhdc is handle to the device which was used for obtaining x-axis metrics
+	// m_yhdc is handle to the device which was used for obtaining y-axis metrics
+	// we have no control over the lifetime of any of
+	// these dc's -- we only use these to check that the metrics and other font info is
+	// uptodate -- they should NEVER be passed to any win32 API
+	
+	HDC				m_hdc;
+	HDC				m_xhdc;
+	HDC				m_yhdc;
+	
 	UT_uint32				m_defaultCharWidth;
 	HFONT                   m_layoutFont;
 	TEXTMETRIC				m_tm;
@@ -92,18 +143,54 @@ private:
 
 	// a cache of 'allocFont *' at a given size
 	mutable UT_Vector		m_allocFonts;
+	bool                    m_bGUIFont;
+	double                  m_fPointSize;
 };
 
 //////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////
 
-class GR_Win32Graphics : public GR_Graphics
+class GR_Win32AllocInfo : public GR_AllocInfo
 {
-public:
-	GR_Win32Graphics(HDC, HWND, XAP_App *);					/* for screen */
-	GR_Win32Graphics(HDC, const DOCINFO *, XAP_App *, HGLOBAL hDevMode = NULL);	/* for printing */
-	~GR_Win32Graphics();
+  public:
+	GR_Win32AllocInfo():
+		m_hdc(0), m_hwnd(0), m_pApp(NULL), m_pDocInfo(NULL), m_hDevMode(NULL) {};
+	
+	GR_Win32AllocInfo(HDC hdc, HWND hwnd, XAP_App * pApp):
+		m_hdc(hdc), m_hwnd(hwnd), m_pApp(pApp), m_pDocInfo(NULL), m_hDevMode(NULL) {};
+	
+	GR_Win32AllocInfo(HDC hdc, const DOCINFO* pDoc, XAP_App * pApp, HGLOBAL devmode):
+		m_hdc(hdc), m_hwnd(0), m_pApp(pApp), m_pDocInfo(pDoc), m_hDevMode(devmode) {};
 
+	virtual GR_GraphicsId getType() const {return GRID_WIN32;}
+	virtual bool isPrinterGraphics() const {return (m_pDocInfo != 0);}
+	
+	HDC               m_hdc;
+	HWND              m_hwnd;
+	XAP_App *         m_pApp;
+	const DOCINFO *   m_pDocInfo;
+	HGLOBAL           m_hDevMode;
+};
+
+
+class ABI_EXPORT GR_Win32Graphics : public GR_Graphics
+{
+	// all constructors are protected; instances must be created via
+	// GR_GraphicsFactory
+public:
+	virtual ~GR_Win32Graphics();
+
+	static UT_uint32 s_getClassId() {return GRID_WIN32;}
+	virtual UT_uint32 getClassId() {return s_getClassId();}
+	
+	virtual GR_Capability getCapability() {return GRCAP_SCREEN_AND_PRINTER;}
+	
+	static const char *    graphicsDescriptor(){return "Win32 Default";}
+	static GR_Graphics *   graphicsAllocator(GR_AllocInfo&);
+
+	static  GR_Graphics *   getPrinterGraphics(const char * pPrinterName,
+											   const char * pDocName);
+	
 	virtual void			drawGlyph(UT_uint32 glyph_idx, UT_sint32 xoff, UT_sint32 yoff);
 	virtual void			drawChar(UT_UCSChar Char, UT_sint32 xoff, UT_sint32 yoff);
 	virtual void			drawChars(const UT_UCSChar* pChars,
@@ -111,17 +198,13 @@ public:
 									  UT_sint32 xoff, UT_sint32 yoff,
 									  int * pCharWidth);
 	virtual void			setFont(GR_Font* pFont);
+	virtual void            clearFont(void) { m_pFont = NULL;}
 	virtual UT_uint32		getFontHeight();
-	virtual UT_uint32		measureUnRemappedChar(const UT_UCSChar c);
+	virtual UT_sint32		measureUnRemappedChar(const UT_UCSChar c);
 	virtual void			setColor(const UT_RGBColor& clr);
 	virtual void            getColor(UT_RGBColor& clr);
 	virtual GR_Font*		getGUIFont();
-	virtual GR_Font*		findFont(const char* pszFontFamily,
-									 const char* pszFontStyle,
-									 const char* pszFontVariant,
-									 const char* pszFontWeight,
-									 const char* pszFontStretch,
-									 const char* pszFontSize);
+
 	virtual UT_uint32		getFontAscent();
 	virtual UT_uint32		getFontDescent();
 	virtual void			getCoverage(UT_NumberVector& coverage);
@@ -176,19 +259,46 @@ public:
 	virtual UT_uint32		getFontAscent(GR_Font *);
 	virtual UT_uint32		getFontDescent(GR_Font *);
 	virtual UT_uint32		getFontHeight(GR_Font *);
+
+    virtual GR_Image * genImageFromRectangle(const UT_Rect & r);
 	virtual void		  saveRectangle(UT_Rect & r, UT_uint32 iIndx);
 	virtual void		  restoreRectangle(UT_uint32 iIndx);
 	virtual void 		  flush(void);
 	void setBrush(HBRUSH hBrush){ m_hClearBrush = hBrush;};
+
+
+	virtual void          setPrintDC(HDC dc);
+	HDC                   getPrintDC() const {return m_printHDC;}
 	
+	double                getXYRatio() const {return m_fXYRatio;}
+	double                getXYRatioPrint() const {return m_fXYRatioPrint;}
+
+	static bool fixDevMode(HGLOBAL hModDev);
 	
 protected:
+	// all instances have to be created via GR_GraphicsFactory; see gr_Graphics.h
+	GR_Win32Graphics(HDC, HWND, XAP_App *);					/* for screen */
+	GR_Win32Graphics(HDC, const DOCINFO *, XAP_App *, HGLOBAL hDevMode = NULL);	/* for printing */
+	
+	BITMAPINFO * ConvertDDBToDIB(HBITMAP bitmap, HPALETTE hPal, DWORD dwCompression);
+
+	virtual GR_Font*		_findFont(const char* pszFontFamily,
+									  const char* pszFontStyle,
+									  const char* pszFontVariant,
+									  const char* pszFontWeight,
+									  const char* pszFontStretch,
+									  const char* pszFontSize);
+
 	virtual UT_uint32 	getDeviceResolution(void) const;
 	void					_setColor(DWORD clrRef);
 
+  private:
+	virtual GR_Win32Font * _newFont(LOGFONT & lf, double fPointSize, HDC hdc, HDC printDC);
+
+  protected:
 
 	HDC						m_hdc;
-	HDC                     m_printerDC;
+	HDC                     m_printHDC;
 	HWND 					m_hwnd;
 	const DOCINFO *			m_pDocInfo;
 	bool					m_bPrint;
@@ -206,6 +316,10 @@ protected:
 
 	DWORD					m_clrCurrent;
 	DWORD					m_3dColors[COUNT_3D_COLORS];
+	int                     m_nPrintLogPixelsY;
+	double                  m_fXYRatio;
+	double                  m_fXYRatioPrint;
+
 
 private:
 	void 					_constructorCommonCode(HDC);
@@ -217,7 +331,7 @@ private:
 
 	UT_UCS2Char*			m_remapBuffer;
 	UT_uint32				m_remapBufferSize;
-	UT_UCS2Char*				m_remapIndices;
+	UT_UCS2Char*			m_remapIndices;
 
 	UT_RGBColor				m_curColor;
 	UT_Vector				m_vSaveRect;
@@ -226,7 +340,6 @@ private:
 	int						m_nLogPixelsY;
 	HGLOBAL					m_hDevMode;
 	
-	
 	typedef struct
 	{
 		HPEN 	hPen;
