@@ -439,29 +439,217 @@ UT_uint32 UT_XML_strncpy(XML_Char * szDest, UT_uint32 nLen, const XML_Char * szS
 
 UT_UCSChar UT_decodeUTF8char(const XML_Char * p, UT_uint32 len)
 {
-	UT_UCSChar ucs, ucs1, ucs2, ucs3;
+	UT_UCSChar ucs, ucs1, ucs2, ucs3, ucs4;
 
 	switch (len)
 	{
-	case 2:
-		ucs1 = static_cast<UT_UCSChar>(p[0] & 0x1f);
-		ucs2 = static_cast<UT_UCSChar>(p[1] & 0x3f);
-		ucs  = (ucs1 << 6) | ucs2;
-		return ucs;
+		case 2:
+			ucs1 = static_cast<UT_UCSChar>(p[0] & 0x1f);
+			ucs2 = static_cast<UT_UCSChar>(p[1] & 0x3f);
+			ucs  = (ucs1 << 6) | ucs2;
+			return ucs;
 
-	case 3:
-		ucs1 = static_cast<UT_UCSChar>(p[0] & 0x0f);
-		ucs2 = static_cast<UT_UCSChar>(p[1] & 0x3f);
-		ucs3 = static_cast<UT_UCSChar>(p[2] & 0x3f);
-		ucs  = (ucs1 << 12) | (ucs2 << 6) | ucs3;
-		return ucs;
+		case 3:
+			ucs1 = static_cast<UT_UCSChar>(p[0] & 0x0f);
+			ucs2 = static_cast<UT_UCSChar>(p[1] & 0x3f);
+			ucs3 = static_cast<UT_UCSChar>(p[2] & 0x3f);
+			ucs  = (ucs1 << 12) | (ucs2 << 6) | ucs3;
+			return ucs;
 
-	default:
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
-		return 0;
+		case 4:
+			ucs1 = static_cast<UT_UCSChar>((p[0] & 0x07) << 2) | static_cast<UT_UCSChar>((p[1] & 0x0f00) >> 4);
+			ucs2 = static_cast<UT_UCSChar>(p[1] & 0x0f);
+			ucs3 = static_cast<UT_UCSChar>(p[2] & 0x3f);
+			ucs4 = static_cast<UT_UCSChar>(p[3] & 0x3f);
+			ucs  = (ucs1 << 16) | (ucs2 << 12) | (ucs3 << 6) | ucs3;
+			return ucs;
+
+		default:
+			UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+			return 0;
 	}
 }
 
+/*! \fn bool UT_isValidXML(const char *s)
+	 \param s The string of characters which is to be checked for XML-validity.
+	 \retval TRUE if the characters are all valid for XML, FALSE if any one of them is not.
+
+	 NB: this function also checks that the string is valid utf-8
+*/
+bool UT_isValidXML(const char *pString)
+{
+	if(!pString)
+		return true;
+
+	if(!UT_isValidUTF8string(pString, strlen(pString)))
+		return false;
+
+	const UT_Byte * s = reinterpret_cast<const UT_Byte *>(pString);
+	
+	while(*s)
+	{
+		if(*s < ' ' && *s != '\t' && *s != '\n' && *s != '\r')
+		{
+			return false;
+		}
+		
+		++s;
+	}
+
+	return true;
+}
+
+/*!
+    XML cannot contain any control characters except \t, \n, \r, see bug 8565
+    (http://www.w3.org/TR/REC-xml/#charsets)
+    
+    This function removes any illegal characters and invalid utf-8 sequences.
+
+    The return value of true indicates that the string was modified
+*/
+bool UT_validXML(char * pString)
+{
+	if(!pString)
+		return false;
+
+	UT_ASSERT(sizeof(XML_Char) == sizeof(UT_Byte));
+	const UT_Byte * p = reinterpret_cast<const UT_Byte *>(pString);	// XML_Char is signed...
+	
+	bool bChanged = false;
+	UT_uint32 len = strlen(pString);
+
+	int bytesInSequence = 0;
+	int bytesExpectedInSequence = 0;
+
+	UT_String s;
+	for (UT_uint32 k=0; k<len; k++)
+	{
+		if (p[k] < 0x80)						// plain us-ascii part of latin-1
+		{
+			if(bytesInSequence != 0)
+				bChanged = true;
+			
+			if(p[k] < ' ' && p[k] >= 0 && p[k] != '\t' && p[k] != '\n' && p[k] != '\r')
+			{
+				bChanged = true;
+			}
+			else
+				s += p[k];
+				
+			bytesInSequence = 0;
+			bytesExpectedInSequence = 0;
+		}
+		else if ((p[k] & 0xf0) == 0xf0)			// lead byte in 4-byte surrogate pair
+		{
+			if(bytesInSequence != 0)
+				bChanged = true;
+			
+			UT_ASSERT_HARMLESS( UT_NOT_IMPLEMENTED );
+			bytesExpectedInSequence = 4;
+			bytesInSequence = 1;
+		}
+		else if ((p[k] & 0xe0) == 0xe0)			// lead byte in 3-byte sequence
+		{
+			if(bytesInSequence != 0)
+				bChanged = true;
+			
+			bytesExpectedInSequence = 3;
+			bytesInSequence = 1;
+		}
+		else if ((p[k] & 0xc0) == 0xc0)			// lead byte in 2-byte sequence
+		{
+			if(bytesInSequence != 0)
+				bChanged = true;
+			
+			bytesExpectedInSequence = 2;
+			bytesInSequence = 1;
+		}
+		else if ((p[k] & 0x80) == 0x80)			// trailing byte in multi-byte sequence
+		{
+			bytesInSequence++;
+			if (bytesInSequence == bytesExpectedInSequence)		// final byte in multi-byte sequence
+			{
+				for(UT_sint32 i = k - bytesInSequence + 1; i <= (UT_sint32)k; i++)
+				{
+					s += p[i];
+				}
+				
+				bytesInSequence = 0;
+				bytesExpectedInSequence = 0;
+			}
+		}
+	}
+
+	strncpy(pString, s.c_str(), s.length());
+	
+	return bChanged;
+}
+
+	
+
+/*!
+    Checks that string p contains valid utf-8 sequence
+*/
+bool  UT_isValidUTF8string(const XML_Char * pString, UT_uint32 len)
+{
+	UT_ASSERT(sizeof(XML_Char) == sizeof(UT_Byte));
+	const UT_Byte * p = reinterpret_cast<const UT_Byte *>(pString);	// XML_Char is signed...
+
+	int bytesInSequence = 0;
+	int bytesExpectedInSequence = 0;
+
+	for (UT_uint32 k=0; k<len; k++)
+	{
+		if (p[k] < 0x80)						// plain us-ascii part of latin-1
+		{
+			if(bytesInSequence != 0)
+				return false;
+
+			bytesExpectedInSequence = 0;
+			bytesInSequence = 0;
+		}
+		else if ((p[k] & 0xf0) == 0xf0)			// lead byte in 4-byte surrogate pair
+		{
+			if(bytesInSequence != 0)
+				return false;
+			
+			bytesExpectedInSequence = 4;
+			bytesInSequence = 1;
+		}
+		else if ((p[k] & 0xe0) == 0xe0)			// lead byte in 3-byte sequence
+		{
+			if(bytesInSequence != 0)
+				return false;
+			
+			bytesExpectedInSequence = 3;
+			bytesInSequence = 1;
+		}
+		else if ((p[k] & 0xc0) == 0xc0)			// lead byte in 2-byte sequence
+		{
+			if(bytesInSequence != 0)
+				return false;
+			
+			bytesExpectedInSequence = 2;
+			bytesInSequence = 1;
+		}
+		else if ((p[k] & 0x80) == 0x80)			// trailing byte in multi-byte sequence
+		{
+			if(bytesInSequence == 0)
+				return false;
+			
+			bytesInSequence++;
+			if (bytesInSequence == bytesExpectedInSequence)		// final byte in multi-byte sequence
+			{
+				bytesInSequence = 0;
+				bytesExpectedInSequence = 0;
+			}
+		}
+	}
+
+	return true;
+}
+	
+
 void UT_decodeUTF8string(const XML_Char * pString, UT_uint32 len, UT_GrowBuf * pResult)
 {
 	// decode the given string [ p[0]...p[len] ] and append to the given growbuf.
@@ -488,7 +676,8 @@ void UT_decodeUTF8string(const XML_Char * pString, UT_uint32 len, UT_GrowBuf * p
 			// mechanism for rare characters in future extensions
 			// of the unicode standard.
 			UT_ASSERT(bytesInSequence == 0);
-			UT_ASSERT(UT_NOT_IMPLEMENTED);
+			bytesExpectedInSequence = 4;
+			buf[bytesInSequence++] = p[k];
 		}
 		else if ((p[k] & 0xe0) == 0xe0)			// lead byte in 3-byte sequence
 		{
@@ -947,8 +1136,8 @@ bool UT_UCS2_isspace(UT_UCS2Char c)
 
 bool UT_UCS2_isalpha(UT_UCS2Char c)
 {
-    FriBidiCharType type = fribidi_get_type(c);
-    return (FRIBIDI_IS_LETTER(type) != 0);
+    UT_BidiCharType type = UT_bidiGetCharType(c);
+    return (UT_BIDI_IS_LETTER(type) != 0);
 };
 
 bool UT_UCS2_isSentenceSeparator(UT_UCS2Char c)
@@ -1132,8 +1321,8 @@ bool UT_UCS4_isspace(UT_UCS4Char c)
 
 bool UT_UCS4_isalpha(UT_UCS4Char c)
 {
-    FriBidiCharType type = fribidi_get_type(c);
-    return (FRIBIDI_IS_LETTER(type) != 0);
+    UT_BidiCharType type = UT_bidiGetCharType(c);
+    return (UT_BIDI_IS_LETTER(type) != 0);
 }
 
 bool UT_UCS4_isSentenceSeparator(UT_UCS4Char c)
@@ -1582,3 +1771,111 @@ const char* std_size_string(float f)
   return string;
 }
 
+#ifndef WIN32
+
+UT_BidiCharType UT_bidiGetCharType(UT_UCS4Char c)
+{
+#ifndef NO_BIDI_SUPPORT
+	return fribidi_get_type(c);
+#else
+	return UT_BIDI_LTR;
+#endif
+}
+
+/*!
+    pStrOut needs to contain space for len characters + terminating 0
+*/
+bool UT_bidiReorderString(const UT_UCS4Char * pStrIn, UT_uint32 len, UT_BidiCharType baseDir,
+						  UT_UCS4Char * pStrOut)
+{
+	UT_return_val_if_fail( pStrIn && pStrOut, false );
+	
+#ifndef NO_BIDI_SUPPORT
+	// this works around 8685; this should be left here, in fact any decent optimising
+	// compiler should remove this code if the bug does not exist
+	if(sizeof(FriBidiChar) > sizeof(UT_UCS4Char))
+	{
+		static FriBidiChar* pFBDC = NULL;
+		static FriBidiChar* pFBDC2 = NULL;
+		static UT_uint32 iFBDlen = 0;
+
+		if(iFBDlen < len + 1)
+		{
+			delete [] pFBDC; delete [] pFBDC2;
+			iFBDlen = 0;
+			
+			pFBDC = new FriBidiChar [len + 1];
+			pFBDC2 = new FriBidiChar [len + 1];
+
+			UT_return_val_if_fail( pFBDC && pFBDC2, false );
+
+			iFBDlen = len + 1;
+		}
+
+		UT_uint32 i;
+		for(i = 0; i < len; ++i)
+		{
+			pFBDC[i] = (FriBidiChar) pStrIn[i];
+		}
+
+		pFBDC[i] = 0;
+
+		int iRet = fribidi_log2vis (pFBDC, len, &baseDir, pFBDC2, NULL, NULL, NULL);
+
+		for(i = 0; i < len; ++i)
+		{
+			pStrOut[i] = (UT_UCS4Char) pFBDC2[i];
+		}
+
+		pStrOut[i] = 0;
+
+		return iRet;
+	}
+	else
+	{
+		return (0 != fribidi_log2vis ((FriBidiChar *)pStrIn, len, &baseDir, (FriBidiChar*)pStrOut, NULL, NULL, NULL));
+	}
+	
+#else
+	if(!pStrIn || !*pStrIn)
+		return true;
+
+	UT_return_val_if_fail( pStrOut, false );
+
+	UT_UCS4_strncpy(pStrOut, pStrIn, len);
+	return true;
+#endif
+}
+
+bool UT_bidiMapLog2Vis(const UT_UCS4Char * pStrIn, UT_uint32 len, UT_BidiCharType baseDir,
+					   UT_uint32 *pL2V, UT_uint32 * pV2L, UT_Byte * pEmbed)
+{
+#ifndef NO_BIDI_SUPPORT
+	// if this assert fails, we have a serious problem ...
+	UT_ASSERT_HARMLESS( sizeof(UT_UCS4Char) == sizeof(FriBidiChar) );
+	return (0 != fribidi_log2vis ((FriBidiChar *)pStrIn, len, &baseDir,
+								  NULL, (FriBidiStrIndex*)pL2V, (FriBidiStrIndex*)pV2L, (FriBidiLevel*)pEmbed));
+#else
+	UT_return_val_if_fail( pL2V && pV2L && pEmbed, false );
+	for(UT_uint32 i = 0; i < len; ++i)
+	{
+		pL2V[i] = i;
+		pV2L[i] = i;
+		pEmbed[i] = 0;
+	}
+
+	return true;
+#endif
+}
+
+bool UT_bidiGetMirrorChar(UT_UCS4Char c, UT_UCS4Char &mc)
+{
+#ifndef NO_BIDI_SUPPORT
+	return (0 != fribidi_get_mirror_char(c, (FriBidiChar*)&mc));
+#else
+	return false;
+#endif
+}
+
+
+#endif
