@@ -26,6 +26,7 @@
 #include "ut_misc.h"
 
 #include "xap_App.h"
+#include "xap_EncodingManager.h"
 #include "xap_Win32App.h"
 #include "xap_Win32FrameImpl.h"
 
@@ -50,9 +51,7 @@ XAP_Win32Dialog_FontChooser::XAP_Win32Dialog_FontChooser(XAP_DialogFactory * pDl
 													 XAP_Dialog_Id id)
 	: XAP_Dialog_FontChooser(pDlgFactory,id),
 	m_pPreviewWidget(NULL),
-	m_bWin32Overline(false),
-	m_bWin32Topline(false),
-	m_bWin32Bottomline(false)
+	m_bWin32Overline(false)
 {
 }
 
@@ -69,6 +68,8 @@ void XAP_Win32Dialog_FontChooser::runModal(XAP_Frame * pFrame)
 
 	XAP_Win32App * pApp = static_cast<XAP_Win32App *>(pFrame->getApp());
 	UT_return_if_fail(pApp);
+	const XAP_EncodingManager *pEncMan = pApp->getEncodingManager();
+	UT_return_if_fail(pEncMan);
 
 	UT_DEBUGMSG(("FontChooserStart: Family[%s] Size[%s] Weight[%s] Style[%s] Color[%s] Underline[%d] StrikeOut[%d]\n",
 				 ((m_pFontFamily) ? m_pFontFamily : ""),
@@ -80,8 +81,6 @@ void XAP_Win32Dialog_FontChooser::runModal(XAP_Frame * pFrame)
 				 (m_bStrikeout)));
 
 	m_bWin32Overline   = m_bOverline;
-	m_bWin32Topline    = m_bTopline;
-	m_bWin32Bottomline = m_bBottomline;
 	m_bWin32Hidden     = m_bHidden;
 	m_bWin32SuperScript = m_bSuperScript;
 	m_bWin32SubScript = m_bSubScript;
@@ -110,7 +109,7 @@ void XAP_Win32Dialog_FontChooser::runModal(XAP_Frame * pFrame)
 	cf.hInstance = pApp->getInstance();
 
 	if (m_pFontFamily && *m_pFontFamily)
-		strcpy(lf.lfFaceName,m_pFontFamily);
+		strcpy(lf.lfFaceName,pEncMan->strToNative(m_pFontFamily, "UTF-8"));
 	else
 		cf.Flags |= CF_NOFACESEL;
 
@@ -153,23 +152,27 @@ void XAP_Win32Dialog_FontChooser::runModal(XAP_Frame * pFrame)
 
 	// run the actual dialog...
 	m_answer = (ChooseFont(&cf) ? a_OK : a_CANCEL);
+	// Convert the font name returned by the Windows Font Chooser
+	// to UTF-8.
+	// (strToNative() puts the string in a static buffer).
+	const char *szFontFamily = pEncMan->strToNative(lf.lfFaceName, "UTF-8", true);
 
 	if (m_answer == a_OK)
 	{
 		if(m_pFontFamily)
 		{
-			if((UT_stricmp(lf.lfFaceName,m_pFontFamily) != 0))
+			if((UT_stricmp(szFontFamily, m_pFontFamily) != 0))
 			{
 				m_bChangedFontFamily = true;
-				CLONEP((char *&) m_pFontFamily, lf.lfFaceName);
+				CLONEP((char *&) m_pFontFamily, szFontFamily);
 			}
 		}
 		else
 		{
-			if(lf.lfFaceName[0])
+			if(szFontFamily[0])
 			{
 				m_bChangedFontFamily = true;
-				CLONEP((char *&) m_pFontFamily, lf.lfFaceName);
+				CLONEP((char *&) m_pFontFamily, szFontFamily);
 			}
 		}
 
@@ -231,18 +234,12 @@ void XAP_Win32Dialog_FontChooser::runModal(XAP_Frame * pFrame)
 		m_bChangedUnderline  = ((lf.lfUnderline == TRUE) != m_bUnderline);
 		m_bChangedStrikeOut  = ((lf.lfStrikeOut == TRUE) != m_bStrikeout);
 		m_bChangedOverline   = (m_bWin32Overline   != m_bOverline);
-		m_bChangedTopline    = (m_bWin32Topline    != m_bTopline);
-		m_bChangedBottomline = (m_bWin32Bottomline != m_bBottomline);
 		if (m_bChangedUnderline ||
             m_bChangedStrikeOut ||
-            m_bChangedOverline  ||
-            m_bChangedTopline   ||
-            m_bChangedBottomline)
+            m_bChangedOverline)
 			setFontDecoration( (lf.lfUnderline == TRUE),
                                 m_bWin32Overline,
-                                (lf.lfStrikeOut == TRUE),
-                                m_bWin32Topline,
-                                m_bWin32Bottomline );
+                                (lf.lfStrikeOut == TRUE), NULL, NULL);
 
 		m_bChangedHidden = (m_bWin32Hidden != m_bHidden);
 		m_bChangedSuperScript = (m_bWin32SuperScript != m_bSuperScript);
@@ -324,8 +321,6 @@ BOOL XAP_Win32Dialog_FontChooser::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM
 	_DS(FONT_BTN_STRIKEOUT,		DLG_UFS_StrikeoutCheck);
 	_DS(FONT_BTN_UNDERLINE,		DLG_UFS_UnderlineCheck);
 	_DS(FONT_CHK_OVERLINE,		DLG_UFS_OverlineCheck);
-	_DS(FONT_CHK_TOPLINE,		DLG_UFS_ToplineCheck);
-	_DS(FONT_CHK_BOTTOMLINE,	DLG_UFS_BottomlineCheck);	
 	_DS(FONT_TEXT_COLOR,		DLG_UFS_ColorLabel);
 	_DS(FONT_TEXT_SCRIPT,		DLG_UFS_ScriptLabel);
 	_DS(FONT_TEXT_SAMPLE,		DLG_UFS_SampleFrameLabel);
@@ -339,12 +334,6 @@ BOOL XAP_Win32Dialog_FontChooser::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM
 	if( m_bWin32Overline )
 		CheckDlgButton( hWnd, XAP_RID_DIALOG_FONT_CHK_OVERLINE, BST_CHECKED );
 
-	if( m_bWin32Topline )
-		CheckDlgButton( hWnd, XAP_RID_DIALOG_FONT_CHK_TOPLINE, BST_CHECKED );
-
-	if( m_bWin32Bottomline )
-		CheckDlgButton( hWnd, XAP_RID_DIALOG_FONT_CHK_BOTTOMLINE, BST_CHECKED );
-
 	if( m_bWin32Hidden )
 		CheckDlgButton( hWnd, XAP_RID_DIALOG_FONT_CHK_HIDDEN, BST_CHECKED );
 		
@@ -385,13 +374,6 @@ BOOL XAP_Win32Dialog_FontChooser::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lP
 		m_bWin32Overline = (IsDlgButtonChecked(hWnd,XAP_RID_DIALOG_FONT_CHK_OVERLINE)==BST_CHECKED);
 		return 1;
 
-	case XAP_RID_DIALOG_FONT_CHK_TOPLINE:
-		m_bWin32Topline = (IsDlgButtonChecked(hWnd,XAP_RID_DIALOG_FONT_CHK_TOPLINE)==BST_CHECKED);
-		return 1;
-
-	case XAP_RID_DIALOG_FONT_CHK_BOTTOMLINE:
-		m_bWin32Bottomline = (IsDlgButtonChecked(hWnd,XAP_RID_DIALOG_FONT_CHK_BOTTOMLINE)==BST_CHECKED);
-
 	case XAP_RID_DIALOG_FONT_CHK_HIDDEN:
 		m_bWin32Hidden = (IsDlgButtonChecked(hWnd,XAP_RID_DIALOG_FONT_CHK_HIDDEN)==BST_CHECKED);
 		return 1;
@@ -414,15 +396,13 @@ BOOL XAP_Win32Dialog_FontChooser::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lP
 	}
 }
 
-extern bool helpLocalizeAndOpenURL(XAP_Frame * pFrame, bool bLocal, const char* pathBeforeLang, const char* pathAfterLang);
+extern bool helpLocalizeAndOpenURL(const char* pathBeforeLang, const char* pathAfterLang, const char *remoteURLbase);
 
 BOOL XAP_Win32Dialog_FontChooser::_callHelp()
 {
-	XAP_Frame *pFrame = m_pApp->findValidFrame();
-	
 	if ( getHelpUrl().size () > 0 )
     {
-		helpLocalizeAndOpenURL ( pFrame, true, "AbiWord/help", getHelpUrl().c_str() ) ;
+		helpLocalizeAndOpenURL ("AbiWord/help", getHelpUrl().c_str(), "http://www.abisource.com/help/" );
     }
 	else
     {
