@@ -20,6 +20,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include "ap_Features.h"
 #include "ut_types.h"
 #include "ut_assert.h"
 #include "ut_string.h"
@@ -621,11 +622,11 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Changes)
 	switch(id)
 	{
 	case AP_MENU_ID_FILE_SAVE:
-	  if (!pView->getDocument()->isDirty() || !pView->canDo(true))
+		if (!pView->getDocument()->isDirty() /*|| !pView->canDo(true)*/) // this caused bug 7580
 	    s = EV_MIS_Gray;
 	  break;
 	case AP_MENU_ID_FILE_REVERT:
-	  if (!pView->getDocument()->isDirty() || !pView->canDo(true))
+		if (!pView->getDocument()->isDirty() /*|| !pView->canDo(true)*/) // this cause bug 7580
 	    s = EV_MIS_Gray;
 	  break;
 	case AP_MENU_ID_EDIT_UNDO:
@@ -1011,6 +1012,8 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_View)
 	case AP_MENU_ID_VIEW_RULER:
 		if ( pFrameData->m_bShowRuler && !pFrameData->m_bIsFullScreen)
 			s = EV_MIS_Toggled;
+		else if( pFrameData->m_bIsFullScreen )
+			s = EV_MIS_Gray;
 		else
 			s = EV_MIS_ZERO;
 		break;
@@ -1029,24 +1032,32 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_View)
 	case AP_MENU_ID_VIEW_TB_1:
 		if ( pFrameData->m_bShowBar[0] && !pFrameData->m_bIsFullScreen)
 			s = EV_MIS_Toggled;
+		else if( pFrameData->m_bIsFullScreen )
+			s = EV_MIS_Gray;
 		else
 			s = EV_MIS_ZERO;
 		break;
 	case AP_MENU_ID_VIEW_TB_2:	
 		if ( pFrameData->m_bShowBar[1] && !pFrameData->m_bIsFullScreen)
 			s = EV_MIS_Toggled;
+		else if( pFrameData->m_bIsFullScreen )
+			s = EV_MIS_Gray;
 		else
 			s = EV_MIS_ZERO;
 		break;
 	case AP_MENU_ID_VIEW_TB_3:	
 		if ( pFrameData->m_bShowBar[2] && !pFrameData->m_bIsFullScreen)
 			s = EV_MIS_Toggled;
+		else if( pFrameData->m_bIsFullScreen )
+			s = EV_MIS_Gray;
 		else
 			s = EV_MIS_ZERO;
 		break;
 	case AP_MENU_ID_VIEW_TB_4:	
 		if ( pFrameData->m_bShowBar[3] && !pFrameData->m_bIsFullScreen)
 			s = EV_MIS_Toggled;
+		else if( pFrameData->m_bIsFullScreen )
+			s = EV_MIS_Gray;
 		else
 			s = EV_MIS_ZERO;
 		break;
@@ -1064,6 +1075,8 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_View)
               if ( pFrameData->m_bShowStatusBar &&
 				   !pFrameData->m_bIsFullScreen )
 				  s = EV_MIS_Toggled;
+			  else if( pFrameData->m_bIsFullScreen )
+				  s = EV_MIS_Gray;
 			  else
 				  s = EV_MIS_ZERO;
 			  break;
@@ -1480,6 +1493,35 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_InImage)
 	return EV_MIS_Gray;
 }
 
+Defun_EV_GetMenuItemState_Fn(ap_GetState_SetPosImage)
+{
+        ABIWORD_VIEW;
+  	UT_return_val_if_fail (pView, EV_MIS_Gray);
+	bool bCont= false;
+	if(pView->isImageSelected())
+	{
+		bCont = true;
+	}
+	if(!bCont & pView->getFrameEdit()->isActive())
+	{
+	        fl_FrameLayout * pFL = pView->getFrameLayout();
+		if(pFL && pFL->getFrameType() == FL_FRAME_TEXTBOX_TYPE)
+		{
+		        return EV_MIS_Gray;
+		}
+		bCont = true;
+	}
+	if(!bCont)
+	{
+	  return EV_MIS_Gray;
+	}
+	if(pView->isHdrFtrEdit() || pView->isInHdrFtr(pView->getPoint()))
+	{
+		return EV_MIS_Gray;
+	}
+	return EV_MIS_ZERO;
+}
+
 Defun_EV_GetMenuItemState_Fn(ap_GetState_InFrame)
 {
 	ABIWORD_VIEW;
