@@ -404,7 +404,7 @@ void s_AbiWord_1_Listener::_openTag(const char * szPrefix, const char * szSuffix
 #else /* ENABLE_RESOURCE_MANAGER */
 	const PP_AttrProp * pAP = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
-	xxx_UT_DEBUGMSG(("_openTag: api %d, bHaveProp %d\n",api, bHaveProp));
+	UT_DEBUGMSG(("_openTag: api %d, bHaveProp %d\n",api, bHaveProp));
 	m_pie->write("<");
 
 	UT_return_if_fail(szPrefix && *szPrefix);
@@ -529,6 +529,59 @@ void s_AbiWord_1_Listener::_openTag(const char * szPrefix, const char * szSuffix
 		}
 		m_pie->write (tag.utf8_str (), tag.byteLength());
 	}
+	else if(UT_strcmp(szPrefix,"embed") == 0)
+	{
+		UT_UTF8String tag;
+		const char * szPropVal = NULL;
+		pAP->getAttribute("dataid",szPropVal);
+		if(szPropVal != NULL)
+		{
+			tag = ">";
+			if (bNewLineAfter) tag += "\n";
+			m_pie->write (tag.utf8_str (), tag.byteLength());
+			tag.clear();
+			tag = "<image dataid=";
+			tag += "\"";
+			tag += "snapshot-png-";
+			tag += szPropVal;
+			tag += "\"";
+			tag += " ";
+			tag += PT_PROPS_ATTRIBUTE_NAME;
+			tag += "=\"";
+			bool bFound = pAP->getProperty("height", szPropVal);
+			UT_UTF8String sVal;
+			if(bFound)
+			{
+				double dInch = static_cast<double>(atoi(szPropVal))/UT_LAYOUT_RESOLUTION;
+				UT_UTF8String_sprintf(sVal,"%fin",dInch);
+				tag += "height:";
+				tag += sVal;
+				tag += "; ";
+			}
+			bFound = pAP->getProperty("width", szPropVal);
+			if(bFound)
+			{
+				double dInch = static_cast<double>(atoi(szPropVal))/UT_LAYOUT_RESOLUTION;
+				UT_UTF8String_sprintf(sVal,"%fin",dInch);
+				tag += "width:";
+				tag += sVal;
+				tag += "\"";
+			}
+			tag += "/";
+			tag += ">";
+			tag += "</embed";
+			tag += ">";
+		}
+		else
+		{
+			if (szSuffix)
+				if (*szSuffix == '/')
+					tag += "/";
+			tag += ">";
+			if (bNewLineAfter) tag += "\n";
+		}
+		m_pie->write (tag.utf8_str (), tag.byteLength());
+	}
 	else
 	{
 		if (szSuffix)
@@ -1172,7 +1225,6 @@ void s_AbiWord_1_Listener::_handleStyles(void)
 {
 	bool bWroteOpenStyleSection = false;
 
-	const char * szName = NULL;
 	const PD_Style * pStyle=NULL;
 	UT_GenericVector<PD_Style *> vecStyles;
 	m_pDocument->getAllUsedStyles(&vecStyles);
@@ -1190,8 +1242,16 @@ void s_AbiWord_1_Listener::_handleStyles(void)
 		_openTag("s","/",true,api,0);
 	}
 
-	for (k=0; m_pDocument->enumStyles(k, &szName, &pStyle); k++)
+	UT_GenericVector<PD_Style*> * pStyles = NULL;
+	m_pDocument->enumStyles(pStyles);
+	UT_return_if_fail( pStyles );
+	UT_uint32 iStyleCount = m_pDocument->getStyleCount();
+	
+	for (k=0; k < iStyleCount; k++)
 	{
+		pStyle = pStyles->getNthItem(k);
+		UT_return_if_fail( pStyle );
+		
 		if (!pStyle->isUserDefined() || (vecStyles.findItem(const_cast<PD_Style*>(pStyle))) >= 0)
 			continue;
 
@@ -1205,6 +1265,8 @@ void s_AbiWord_1_Listener::_handleStyles(void)
 		_openTag("s","/",true,api,0);
 	}
 
+	delete pStyles;
+	
 	if (bWroteOpenStyleSection)
 		m_pie->write("</styles>\n");
 
