@@ -194,13 +194,6 @@ static void AbiMathView_addToMenus()
     // on in our for loop.  Take a look near the bottom.
     EV_Menu_ActionSet* pActionSet = pApp->getMenuActionSet();
 
-    
-    // We need to go through and add the menu element to each "frame" 
-    // of the application.  We can iterate through the frames by doing
-    // XAP_App::getFrameCount() to tell us how many frames there are,
-    // then calling XAP_App::getFrame(i) to get the i-th frame.
-
-    int frameCount = pApp->getFrameCount();
     XAP_Menu_Factory * pFact = pApp->getMenuFactory();
 
 // Put it after Insert Picture in the Main menu
@@ -296,12 +289,7 @@ static void AbiMathView_addToMenus()
 
     pActionSet->addAction(myEndEquationAction);
     
-    for(int i = 0;i < frameCount;++i)
-    {
-        // Get the current frame that we're iterating through.
-		XAP_Frame* pFrame = pApp->getFrame(i);
-		pFrame->rebuildMenus();
-    }
+    pApp->rebuildMenus();
 }
 
 static void
@@ -317,19 +305,13 @@ AbiMathView_removeFromMenus ()
 	DELETEP( pEM ) ;
 
 	// now remove crap from the menus
-	int frameCount = pApp->getFrameCount();
 	XAP_Menu_Factory * pFact = pApp->getMenuFactory();
 
 	pFact->removeMenuItem("Main",NULL,AbiMathView_MenuLabelEquation);
 	pFact->removeMenuItem("Main",NULL,AbiMathView_MenuLabelFileInsert);
 	pFact->removeMenuItem("Main",NULL,AbiMathView_MenuLabelLatexInsert);
 
-	for (int i = 0; i < frameCount; ++i)
-	{
-		// Get the current frame that we're iterating through.
-		XAP_Frame* pFrame = pApp->getFrame(i);
-		pFrame->rebuildMenus();
-	}
+	pApp->rebuildMenus();
 }
  
 
@@ -613,9 +595,18 @@ void GR_MathManager::initialize(void)
 #endif
 
 	// add the configuration paths
-	Configuration::addConfigurationPath( userConfDir.utf8_str() );
-	Configuration::addConfigurationPath( libConfDir.utf8_str() );
  
+#ifdef XP_TARGET_COCOA
+	if (const char * resources = getenv("ABIWORD_COCOA_BUNDLED_RESOURCES"))
+	{
+		UT_UTF8String bundleConfDir(resources);
+		bundleConfDir += "/gtkmathview/gtkmathview.conf.xml";
+		Configuration::addConfigurationPath(bundleConfDir.utf8_str());
+	}
+#endif
+	Configuration::addConfigurationPath( libConfDir.utf8_str() );
+	Configuration::addConfigurationPath( userConfDir.utf8_str() );
+
 	SmartPtr<AbstractLogger> logger = Logger::create();
 	m_pLogger = logger;
 	SmartPtr<Configuration> configuration = initConfiguration<libxml2_MathView>(logger, getenv("GTKMATHVIEWCONF"));
@@ -624,7 +615,16 @@ void GR_MathManager::initialize(void)
 	// add the dictionary paths
 	configuration->add("dictionary/path", userDictDir.utf8_str());
 	configuration->add("dictionary/path", libDictDir.utf8_str());
-	 
+
+#ifdef XP_TARGET_COCOA
+	if (const char * resources = getenv("ABIWORD_COCOA_BUNDLED_RESOURCES"))
+	{
+		UT_UTF8String bundleDictDir(resources);
+		bundleDictDir += "/gtkmathview/dictionary.xml";
+		configuration->add("dictionary/path", bundleDictDir.utf8_str());
+	}
+#endif
+
 	SmartPtr<GR_Abi_MathGraphicDevice> mathGraphicDevice 
 			= GR_Abi_MathGraphicDevice::create(logger, configuration, getGraphics());
 	m_pMathGraphicDevice = mathGraphicDevice;
@@ -873,8 +873,8 @@ void GR_MathManager::releaseEmbedView(UT_sint32 uid)
 
 extern "C"
 {
-	extern void yyrestart(FILE *input_file);
-	extern int yyparse(char **str);
+	extern void itex2MML_yyrestart(FILE *input_file);
+	extern int  itex2MML_yyparse(char **str);
 }
 
 bool GR_MathManager::convert(UT_uint32 iConType, UT_ByteBuf & From, UT_ByteBuf & To)
@@ -901,15 +901,15 @@ bool GR_MathManager::convert(UT_uint32 iConType, UT_ByteBuf & From, UT_ByteBuf &
 	fwrite(sLatex.utf8_str(), sLatex.size(), 1, tmp);
 	fflush(tmp);
 	rewind(tmp);
-	yyrestart(tmp);
+	itex2MML_yyrestart(tmp);
 
 	char *mathml = 0;
-	int res = yyparse(&mathml);
+	int res = itex2MML_yyparse(&mathml);
 	fclose(tmp);
 	
 	if (res != 0)
 	{
-		pFrame->showMessageBox("TODO: give proper error message!\n",
+		pFrame->showMessageBox("itex2MML failed to convert the LaTeX equation into MathML, sorry!\n", // TODO: fix message
 			XAP_Dialog_MessageBox::b_O, 
 			XAP_Dialog_MessageBox::a_OK);
 		return false;
@@ -917,10 +917,10 @@ bool GR_MathManager::convert(UT_uint32 iConType, UT_ByteBuf & From, UT_ByteBuf &
 
 	UT_UTF8String sMathML(mathml);
 	FREEP(mathml);
-	
+
 	if (sMathML.size() == 0)
 	{
-		UT_UTF8String sErrMessage = "Error converting: <todo>\n";
+	  UT_UTF8String sErrMessage = "itex2MML conversion from LaTex equation resulted in zero-length MathML!\n"; // TODO: fix message
 		//sErrMessage += sLatex;
 		sErrMessage += "\n";
 		// sErrMessage += FullLine;
