@@ -26,23 +26,17 @@
 
 // Internal includes
 #include "OD_Office_Styles.h"
+#include "OD_Style_List.h"
 #include "OD_Style_Style.h"
 #include "OD_Style_MasterPage.h"
 #include "OD_ListenerStateAction.h"
+#include "OD_ListLevelStyle.h"
+#include "OD_StartTag.h"
+#include "OD_ElementStack.h"
 
 // AbiWord includes
 #include <ut_misc.h>
 #include <pd_Document.h>
-#include <ie_impGraphic.h>
-#include <fg_GraphicRaster.h>
-
-// External includes
-#include <glib-object.h>
-#include <gsf/gsf-input-stdio.h>
-#include <gsf/gsf-infile.h>
-#include <gsf/gsf-infile-zip.h>
-
-
 
 
 /**
@@ -50,24 +44,24 @@
  */
 OD_TextContent_ListenerState::OD_TextContent_ListenerState (
                 PD_Document* pDocument,
-                GsfInfile* pGsfInfile,
-                OD_Office_Styles* pStyles)
-                : OD_ListenerState("TextContent"),
+                OD_Office_Styles* pStyles,
+                OD_ElementStack& rElementStack)
+                : OD_ListenerState("TextContent", rElementStack),
                   m_pAbiDocument ( pDocument ),
                   m_pStyles(pStyles),
-                  m_pGsfInfile(pGsfInfile),
                   m_bAcceptingText(false),
                   m_bInSection(false),
-                  m_imgCnt(0),
                   m_row(0),
                   m_col(0),
                   m_cel(0),
                   m_bInTOC(false),
-                  m_bOnContentStream(false)
+                  m_bOnContentStream(false),
+                  m_pCurrentListStyle(NULL),
+                  m_listLevel(0),
+                  m_alreadyDefinedAbiParagraphForList(false)
 {
     UT_ASSERT(m_pAbiDocument);
     UT_ASSERT(m_pStyles);
-    UT_ASSERT(m_pGsfInfile);
 }
 
 
@@ -81,7 +75,6 @@ void OD_TextContent_ListenerState::startElement (const XML_Char* pName,
                                           const XML_Char** ppAtts,
                                           OD_ListenerStateAction& rAction)
 {
-    
     // if we're inside a TOC, ignore the contents since ODT
     // includes a copy of the TOC
     if (m_bInTOC) {
@@ -93,56 +86,18 @@ void OD_TextContent_ListenerState::startElement (const XML_Char* pName,
         const XML_Char* pStyleName = UT_getAttribute ("text:style-name", ppAtts);
         const OD_Style_Style* pStyle = m_pStyles->getSectionStyle(pStyleName,
                                                         m_bOnContentStream);
+                                                        
+        UT_UTF8String props;
+        
+        pStyle->getAbiPropsAttrString(props);
 
-        _insureInSection(pStyle->getAbiPropsAttrString().utf8_str());
+        _insureInSection(props.utf8_str());
 
     } else if (!UT_strcmp(pName, "text:p" ) || !UT_strcmp(pName, "text:h" )) {
-
-        const XML_Char* pStyleName;
-
-        pStyleName = UT_getAttribute ("text:style-name", ppAtts);
-        const OD_Style_Style* pStyle;
-
-        if (pStyleName) {
-            pStyle = m_pStyles->getParagraphStyle(pStyleName, m_bOnContentStream);
-            UT_ASSERT(pStyle);
-        } else {
-            // Use the default style
-            pStyle = m_pStyles->getDefaultParagraphStyle();
-        }
-
-        // Do we have an AbiWord section?
-        _insureInSection();
         
-        // Append an empty paragraph with this one char
-        if (pStyle->getPageBreakBefore()) {
-            m_pAbiDocument->appendStrux(PTX_Block, NULL);
-            UT_UCSChar ucs = (pStyle->getColBreakBefore () ? UCS_VTAB : UCS_FF);
-            m_pAbiDocument->appendSpan (&ucs, 1);
-        }
-
-
-
-        const XML_Char *ppParagraphAtts[3];
+        // It's so big that it deserves its own function.
+        _startParagraphElement(pName, ppAtts, rAction);
         
-        if (pStyle->isAutomatic()) {
-            // Automatic styles are not defined on the document, so, we
-            // just paste its properties.
-            ppParagraphAtts[0] = "props";
-            ppParagraphAtts[1] = pStyle->getAbiPropsAttrString().utf8_str();
-            ppParagraphAtts[2] = 0;
-        } else {
-            // We refer to the style
-            ppParagraphAtts[0] = "style";
-            ppParagraphAtts[1] = pStyle->getDisplayName().utf8_str();
-            ppParagraphAtts[2] = 0;
-        }
-        
-        m_pAbiDocument->appendStrux(PTX_Block, (const XML_Char**)ppParagraphAtts);
-        
-        // We now accept text
-        m_bAcceptingText = true;
-
     } else if (!UT_strcmp(pName, "text:s")) {
         // A number of consecutive white-space characters.
         
@@ -151,8 +106,17 @@ void OD_TextContent_ListenerState::startElement (const XML_Char* pName,
         UT_UCS4String string;
         
         pSpaceCount = UT_getAttribute("text:c", ppAtts);
-        i = sscanf(pSpaceCount, "%d", &spaceCount);
-        UT_ASSERT(i==1);
+        
+        if (pSpaceCount) {
+            i = sscanf(pSpaceCount, "%d", &spaceCount);
+            UT_ASSERT(i==1);
+        } else {
+            // From the OpenDocument specification:
+            // "A missing text:c attribute is interpreted as meaning a
+            // single SPACE character."
+            spaceCount = 1;
+        }
+        
         
         // TODO: A faster (wiser) implementation can be done, I think. (Daniel d'Andrada)
         string.clear();
@@ -195,37 +159,39 @@ void OD_TextContent_ListenerState::startElement (const XML_Char* pName,
         const XML_Char* pStyleName = UT_getAttribute("text:style-name", ppAtts);
         const OD_Style_Style* pStyle;
         
-        if (pStyleName) {
-            pStyle = m_pStyles->getTextStyle(pStyleName, m_bOnContentStream);
-            UT_ASSERT(pStyle);
-        } else {
+        if (!pStyleName) {
             // I haven't seen default styles for "text" family.
             UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
-        }
-        
-        if (!pStyle) {
-            // If it doens't specify a style it's beacause it does specify a
-            // style class an we don't support style classes yet.
-            UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+            
         } else {
-        
-            const XML_Char* ppStyAttr[3];
-            bool ok;
+            pStyle = m_pStyles->getTextStyle(pStyleName, m_bOnContentStream);
             
-            if (pStyle->isAutomatic()) {
-                // It goes "hardcoded"
-                ppStyAttr[0] = "props";
-                ppStyAttr[1] = pStyle->getAbiPropsAttrString().utf8_str();
-                ppStyAttr[2] = 0;
+            if (pStyle) {
+            
+                const XML_Char* ppStyAttr[3];
+                bool ok;
+                UT_UTF8String props;
+                
+                if (pStyle->isAutomatic()) {
+                    pStyle->getAbiPropsAttrString(props);
+                    
+                    // It goes "hardcoded"
+                    ppStyAttr[0] = "props";
+                    ppStyAttr[1] = props.utf8_str();
+                    ppStyAttr[2] = 0;
+                } else {
+                    ppStyAttr[0] = "style";
+                    ppStyAttr[1] = pStyle->getDisplayName().utf8_str();
+                    ppStyAttr[2] = 0;                
+                }
+                
+                _pushInlineFmt(ppStyAttr);
+                ok = m_pAbiDocument->appendFmt(&m_vecInlineFmt);
+                UT_ASSERT(ok);
+                
             } else {
-                ppStyAttr[0] = "style";
-                ppStyAttr[1] = pStyle->getDisplayName().utf8_str();
-                ppStyAttr[2] = 0;                
+                // We just ignore this <text:span>.
             }
-            
-            _pushInlineFmt(ppStyAttr);
-            ok = m_pAbiDocument->appendFmt(&m_vecInlineFmt);
-            UT_ASSERT(ok);
         }
         
     } else if (!UT_strcmp(pName, "text:line-break")) {
@@ -248,12 +214,6 @@ void OD_TextContent_ListenerState::startElement (const XML_Char* pName,
         _insertBookmark (pName, "start");
         _insertBookmark (pName, "end");
         
-    } else if (!UT_strcmp(pName, "draw:image")) {
-        
-        _flush ();
-        _insureInBlock(NULL);
-        _insertImage (ppAtts);
-        
     } else if (!UT_strcmp(pName, "table:table")) {
         
         _insureInSection(NULL);
@@ -334,9 +294,46 @@ void OD_TextContent_ListenerState::startElement (const XML_Char* pName,
         
     } else if (!UT_strcmp(pName, "office:text")) {
         m_bOnContentStream = true;
+        
+    } else if (!UT_strcmp(pName, "text:list")) {
+        
+        if (m_pCurrentListStyle != NULL) {
+            m_listLevel++;
+        } else {
+            const XML_Char* pVal;
+            
+            pVal = UT_getAttribute("text:style-name", ppAtts);
+            
+            
+            
+            m_pCurrentListStyle = m_pStyles->getList(pVal);
+            UT_ASSERT(m_pCurrentListStyle != NULL);
+            
+            m_listLevel = 1;
+        }
+        
+    } else if (!UT_strcmp(pName, "text:list-item")) {
+        m_alreadyDefinedAbiParagraphForList = false;
+        
+    } else if (!UT_strcmp(pName, "draw:frame")) {
+        if (!UT_strcmp(m_rElementStack.getStartTag(0)->getName(), "text:p")) {
+            // A paragraph anchored frame.
+            // It's postponed because AbiWord uses paragraph frames *after* the
+            // paragraph itself but OpenDocument uses *before* its content.
+            rAction.postponeElementParsing("Frame");
+            
+        } else if (!UT_strcmp(m_rElementStack.getStartTag(0)->getName(), "office:text")) {
+            
+            // A page anchored frame.
+            // It's postponed because in AbiWord there must be at least one
+            // paragraph defined before any frame definition.
+            rAction.postponeElementParsing("Frame");
+            
+        } else {
+            UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+            // TODO: Figure out what to do then.
+        }
     }
-    
-    m_elementStack.startElement(pName, ppAtts);
 }
 
 
@@ -357,6 +354,9 @@ void OD_TextContent_ListenerState::endElement (const XML_Char* pName,
         _flush ();
         m_bAcceptingText = false;
         
+        // Bring back the possible postponed parsing of a <draw:frame>
+        rAction.bringUpMostRecentlyPostponedElement("Frame", true);
+        
     } else if (!UT_strcmp(pName, "text:table-of-content")) {
         
         m_bInTOC = false;
@@ -411,7 +411,8 @@ void OD_TextContent_ListenerState::endElement (const XML_Char* pName,
         
         // We were inside a <office:text> element.
         
-        // We can now bring up the postponed parsing (headers and footers)
+        // We can now bring up the postponed parsing (headers/footers and
+        // page-anchored frames)
         rAction.bringUpPostponedElements(false);
         
     } else if (!UT_strcmp(pName, "style:header") ||
@@ -424,9 +425,14 @@ void OD_TextContent_ListenerState::endElement (const XML_Char* pName,
         UT_ASSERT(!m_bOnContentStream);
         
         rAction.popState();
+        
+    } else if (!UT_strcmp(pName, "text:list")) {
+        m_listLevel--;
+        if (m_listLevel == 0) {
+            m_pCurrentListStyle = NULL;
+        }
+        
     }
-    
-    m_elementStack.endElement(pName);
 }
 
 
@@ -442,94 +448,6 @@ void OD_TextContent_ListenerState::charData (
 }
 
 
-/**
- * 
- */
-void OD_TextContent_ListenerState::_insertImage (const XML_Char** ppAtts)
-{
-    UT_Error error      = UT_OK;
-    const XML_Char * width  = UT_getAttribute ("svg:width", ppAtts);
-    const XML_Char * height = UT_getAttribute ("svg:height", ppAtts);
-    const XML_Char * href   = UT_getAttribute ("xlink:href", ppAtts);
-
-    m_imgCnt++;
-
-    UT_ByteBuf img_buf;
-
-    GsfInfile * pictures_dir =
-        GSF_INFILE(gsf_infile_child_by_name(m_pGsfInfile, "Pictures"));
-
-
-    // 9 == strlen("Pictures/");
-    error = _loadStream(pictures_dir, href+9, img_buf);
-    g_object_unref (G_OBJECT (pictures_dir));
-
-
-    if (error != UT_OK) {
-        return;
-    }
-
-    const char * mimetype   = UT_strdup ("image/png");
-    IE_ImpGraphic * importer    = 0;
-    FG_Graphic* pFG     = 0;
-    UT_ByteBuf * pictData       = 0;
-
-    UT_String propBuffer;
-    UT_String propsName;
-
-    error = IE_ImpGraphic::constructImporter (&img_buf, IEGFT_Unknown, &importer);
-
-    if ((error != UT_OK) || !importer) {
-        FREEP(mimetype);
-        goto Cleanup;
-    }
-
-    error = importer->importGraphic(&img_buf, &pFG);
-    if ((error != UT_OK) || !pFG) {
-        // pictData is already freed in ~FG_Graphic
-        FREEP(mimetype);
-        goto Cleanup;
-    }
-
-    // TODO: can we get back a vector graphic?
-    pictData = static_cast<FG_GraphicRaster *>(pFG)->getRaster_PNG();
-
-    if (!pictData) {
-        // i don't think that this could ever happen, but...
-        FREEP(mimetype);
-        error = UT_ERROR;
-        goto Cleanup;
-    }
-
-    //
-    // This next bit of code will set up our properties based on the image attributes
-    //
-
-    UT_String_sprintf(propBuffer, "width:%s; height:%s", width, height);
-    UT_String_sprintf(propsName, "image%d", m_imgCnt);
-
-    const XML_Char* propsArray[5];
-    propsArray[0] = (XML_Char *)"props";
-    propsArray[1] = (XML_Char *)propBuffer.c_str();
-    propsArray[2] = (XML_Char *)"dataid";
-    propsArray[3] = (XML_Char *)propsName.c_str();
-    propsArray[4] = 0;
-
-    if (!m_pAbiDocument->appendObject (PTO_Image, propsArray)) {
-        FREEP(mimetype);
-        goto Cleanup;
-    }
-
-    if (!m_pAbiDocument->createDataItem(propsName.c_str(), false,
-    pictData, (void*)mimetype, NULL)) {
-        goto Cleanup;
-    }
-
-    Cleanup:
-    DELETEP(importer);
-}
-
-
 /**
  * 
  */
@@ -699,39 +617,6 @@ void OD_TextContent_ListenerState::_popInlineFmt(void)
 }
 
 
-/**
- * 
- */
-UT_Error OD_TextContent_ListenerState::_loadStream ( GsfInfile * oo,
-                 const char * stream,
-                 UT_ByteBuf & buf )
-{
-    guint8 const *data = NULL;
-    size_t len = 0;
-    static const size_t BUF_SZ = 4096;
-  
-    buf.truncate (0);
-    GsfInput * input = gsf_infile_child_by_name(oo, stream);
-
-    if (!input)
-        return UT_ERROR;
-  
-    if (gsf_input_size (input) > 0) {
-        while ((len = gsf_input_remaining (input)) > 0) {
-            len = UT_MIN (len, BUF_SZ);
-            if (NULL == (data = gsf_input_read (input, len, NULL))) {
-                g_object_unref (G_OBJECT (input));
-                return UT_ERROR;
-            }
-            buf.append ((const UT_Byte *)data, len);
-        }
-    }
-  
-    g_object_unref (G_OBJECT (input));
-    return UT_OK;
-}
-
-
 /**
  * Makes sure that an AbiWord section have already been created. Unlike
  * OpenDocument, AbiWord can't have paragraphs without a section to hold them.
@@ -813,3 +698,152 @@ void OD_TextContent_ListenerState::_insureInBlock(const XML_Char ** atts)
         m_bAcceptingText = true;
     }
 }
+
+
+/**
+ * Process <text:p> and <text:h> startElement calls
+ */
+void OD_TextContent_ListenerState::_startParagraphElement (const XML_Char* pName,
+                                          const XML_Char** ppParagraphAtts,
+                                          OD_ListenerStateAction& rAction) {
+        bool bIsListParagraph = false;
+        
+        if (!UT_strcmp(m_rElementStack.getStartTag(0)->getName(), "text:list-item")) {
+            // That's a list paragraph.
+            bIsListParagraph = true;
+        }
+
+        const XML_Char* pStyleName;
+
+        pStyleName = UT_getAttribute ("text:style-name", ppParagraphAtts);
+        const OD_Style_Style* pStyle;
+
+        if (pStyleName) {
+            pStyle = m_pStyles->getParagraphStyle(pStyleName, m_bOnContentStream);
+            UT_ASSERT(pStyle);
+        } else {
+            // Use the default style
+            pStyle = m_pStyles->getDefaultParagraphStyle();
+        }
+        
+        // Do we have an AbiWord section?
+        _insureInSection();
+        
+        // Append an empty paragraph with this one char
+        if (pStyle->getPageBreakBefore()) {
+            m_pAbiDocument->appendStrux(PTX_Block, NULL);
+            UT_UCSChar ucs = (pStyle->getColBreakBefore () ? UCS_VTAB : UCS_FF);
+            m_pAbiDocument->appendSpan (&ucs, 1);
+        }
+
+
+
+        const XML_Char *ppAtts[50];
+        UT_uint8 i = 0;
+        XML_Char listLevel[10];
+        bool ok;
+        UT_UTF8String props;
+        
+        
+        if (bIsListParagraph && !m_alreadyDefinedAbiParagraphForList) {
+            OD_ListLevelStyle* pListLevelStyle;
+            
+            m_alreadyDefinedAbiParagraphForList = true;
+            
+            pListLevelStyle = m_pCurrentListStyle->getLevelStyle(m_listLevel);
+            
+            sprintf(listLevel, "%u", m_listLevel);
+            
+            ppAtts[i++] = "level";
+            ppAtts[i++] = listLevel;
+            ppAtts[i++] = "listid";
+            ppAtts[i++] = pListLevelStyle->getAbiListID()->utf8_str();
+            
+            // Is this really necessary? Because we already have this info on
+            // the <l> tag.
+            ppAtts[i++] = "parentid";
+            ppAtts[i++] = pListLevelStyle->getAbiListParentID()->utf8_str();
+                
+            if (pStyle->isAutomatic()) {
+                // Automatic styles are not defined on the document, so, we
+                // just paste its properties.
+                pStyle->getAbiPropsAttrString(props);
+                
+            } else {
+                // We refer to the style
+                ppAtts[i++] = "style";
+                ppAtts[i++] = pStyle->getDisplayName().utf8_str();
+            }
+
+
+            pListLevelStyle->getAbiProperties(props, pStyle);
+                
+            ppAtts[i++] = "props";
+            ppAtts[i++] = props.utf8_str();
+                
+            ppAtts[i] = 0; // Marks the end of the attributes list.
+            ok = m_pAbiDocument->appendStrux(PTX_Block,
+                                             (const XML_Char**)ppAtts);
+            UT_ASSERT(ok);
+            
+
+            ppAtts[0] = "type";
+            ppAtts[1] = "list_label";
+            ppAtts[2] = 0;
+            ok = m_pAbiDocument->appendObject(PTO_Field, ppAtts);
+            UT_ASSERT(ok);
+            
+            // Inserts a tab character. AbiWord seens to need it in order to
+            // implement the space between the list mark (number/bullet) and
+            // the list text.
+            UT_UCS4String string = "\t";
+            _flush();
+            m_pAbiDocument->appendSpan(string.ucs4_str(), string.size());
+            
+        } else if (bIsListParagraph && m_alreadyDefinedAbiParagraphForList) {
+            // OpenDocument supports multiples paragraphs on a single list item,
+            // But AbiWord works differently. So, we will put a <br/> instead
+            // of adding a new paragraph.
+            
+            UT_UCSChar ucs = UCS_LF;
+            m_pAbiDocument->appendSpan(&ucs,1);
+            
+            
+            if (pStyle->isAutomatic()) {
+                // Automatic styles are not defined on the document, so, we
+                // just paste its properties.
+                pStyle->getAbiPropsAttrString(props);
+                ppAtts[i++] = "props";
+                ppAtts[i++] = props.utf8_str();
+            } else {
+                // We refer to the style
+                ppAtts[i++] = "style";
+                ppAtts[i++] = pStyle->getDisplayName().utf8_str();
+            }
+            ppAtts[i] = 0; // Marks the end of the attributes list.
+            
+            ok = m_pAbiDocument->appendFmt(ppAtts);
+            UT_ASSERT(ok);
+
+        } else {
+            
+            if (pStyle->isAutomatic()) {
+                // Automatic styles are not defined on the document, so, we
+                // just paste its properties.
+                pStyle->getAbiPropsAttrString(props);
+                ppAtts[i++] = "props";
+                ppAtts[i++] = props.utf8_str();
+            } else {
+                // We refer to the style
+                ppAtts[i++] = "style";
+                ppAtts[i++] = pStyle->getDisplayName().utf8_str();
+            }
+        
+            ppAtts[i] = 0; // Marks the end of the attributes list.
+            m_pAbiDocument->appendStrux(PTX_Block, (const XML_Char**)ppAtts);
+        
+        }
+
+        // We now accept text
+        m_bAcceptingText = true;
+}
