@@ -1233,10 +1233,15 @@ public:
 	const XML_Char * abi_sty = _mapStyle(oo_sty);
 
 	const XML_Char * props[3];
-	props[0] = "props";
-	props[1] = abi_sty;
-	props[2] = 0;
-	
+
+	if(abi_sty && *abi_sty) {
+	  props[0] = "props";
+	  props[1] = abi_sty;
+	  props[2] = 0;
+	} else {
+	  props[0] = 0;
+	}
+	  
 	_pushInlineFmt(props);
 	getDocument()->appendFmt(&m_vecInlineFmt);
       }
